//----------------------------------------------------------------------------
//  Project RadioODA
//  Curt Swartzendruber Consulting Services
//  Copyright  1997. All Rights Reserved.
//
//  SUBSYSTEM:    RadioODA Application
//  FILE:         rdodaapp.cpp
//  AUTHOR:       Curt Swartzendruber
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TRadioODAApp (TApplication).
//
//----------------------------------------------------------------------------

#include <owl/pch.h>

#include <stdio.h>

#include "rdodaapp.h"
#include "rdodawnd.h"                        // Definition of client class.
#include "rdodaabd.h"                        // Definition of about dialog.


//{{TRadioODAApp Implementation}}


//
// Build a response table for all messages/commands handled
// by the application.
//
DEFINE_RESPONSE_TABLE1(TRadioODAApp, TApplication)
//{{TRadioODAAppRSP_TBL_BEGIN}}
  EV_COMMAND(CM_FILENEW, CmFileNew),
  EV_COMMAND(CM_FILEOPEN, CmFileOpen),
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
//{{TRadioODAAppRSP_TBL_END}}
END_RESPONSE_TABLE;


//--------------------------------------------------------
// TRadioODAApp
//
TRadioODAApp::TRadioODAApp() : TApplication("Owner Drawing TRadioButton")
{

  // Common file file flags and filters for Open/Save As dialogs.  Filename and directory are
  // computed in the member functions CmFileOpen, and CmFileSaveAs.
  //
  FileData.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
  FileData.SetFilter("All Files (*.*)|*.*|");
  FileData.DefExt = "txt";

  // INSERT>> Your constructor code here.
}


TRadioODAApp::~TRadioODAApp()
{
  // INSERT>> Your destructor code here.
}


//--------------------------------------------------------
// TRadioODAApp
// ~~~~~
// Application intialization.
//
void TRadioODAApp::InitMainWindow()
{
  if (nCmdShow != SW_HIDE)
    nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;

  TSDIDecFrame* frame = new TSDIDecFrame(0, GetName(), 0, false);

  // Assign icons for this application.
  //
  frame->SetIcon(this, IDI_SDIAPPLICATION);
  frame->SetIconSm(this, IDI_SDIAPPLICATION);

  // Menu associated with window and accelerator table associated with table.
  //
  frame->AssignMenu(IDM_SDI);

  // Associate with the accelerator table.
  //
  frame->Attr.AccelTable = IDM_SDI;

  SetMainWindow(frame);

  frame->SetMenuDescr(TMenuDescr(IDM_SDI));
}



//--------------------------------------------------------
// TRadioODAApp
// ~~~~~~~~~~~
// Menu File New command
//
void TRadioODAApp::CmFileNew()
{
}


//--------------------------------------------------------
// TRadioODAApp
// ~~~~~~~~~~~
// Menu File Open command
//
void TRadioODAApp::CmFileOpen()
{
  // Display standard Open dialog box to select a file name.
  //
  *FileData.FileName = 0;

  TRadioODAWindow* client = TYPESAFE_DOWNCAST(GetMainWindow()->GetClientWindow(), TRadioODAWindow);     // Client window for the frame.
  if (client->CanClose())
    if (TFileOpenDialog(GetMainWindow(), FileData).Execute() == IDOK)
      OpenFile();
}


void TRadioODAApp::OpenFile(const char* fileName)
{
  if (fileName)
    strcpy(FileData.FileName, fileName);

}


//{{TSDIDecFrame Implementation}}


TSDIDecFrame::TSDIDecFrame(TWindow* parent, const char far* title, TWindow* clientWnd, bool trackMenuSelection, TModule* module)
:
  TDecoratedFrame(parent, title, !clientWnd ? new TRadioODAWindow(0, "") : clientWnd, trackMenuSelection, module)
{
  // INSERT>> Your constructor code here.

}


TSDIDecFrame::~TSDIDecFrame()
{
  // INSERT>> Your destructor code here.

}


//--------------------------------------------------------
// TRadioODAApp
// ~~~~~~~~~~~
// Menu Help About RadioODA command
//
void TRadioODAApp::CmHelpAbout()
{
  // Show the modal dialog.
  //
  TRadioODAAboutDlg(MainWindow).Execute();
}



int OwlMain(int , char* [])
{
  TRadioODAApp   app;
  return app.Run();
}
