//----------------------------------------------------------------------------
//  Project Ralph Calculus Concepts
//  iMedia Productions, Inc.
//  Copyright  1997. All Rights Reserved.
//
//  SUBSYSTEM:    Generic Application
//  FILE:         ssound.cpp
//  AUTHOR:       Curt M. Swartzendruber, Consultant (swartzc@palmnet.net)
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of SSound (TWindow).
//
//----------------------------------------------------------------------------
#include <dir.h>
#include <stdio.h>
#include <owl/pch.h>
#include "ssound.h"
#include "miscfunc.h"	// Misc functions (LocateFileWithinPath())
#include "serror.h"		// Definition for SError SHARED class

extern char ExePath[];
HWND GLOBAL_HWndCmsSound = NULL;

//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(SSound, TWindow)
//{{TSoundTstDlgClientRSP_TBL_BEGIN}}
	EV_MESSAGE(WM_CMSSOUND_NOTIFYPLAYFINISHED, WmNotifyWhenPlayFinished),
   EV_MESSAGE(WM_CMSSOUND_NOTIFYSTOPPED, WmNotifyWhenStopped),
   EV_MESSAGE(WM_CMSSOUND_INITIATECONTACT, WmCmsSndInitContact),
   EV_MESSAGE(WM_CMSSOUND_PING, WmPing),
//{{TSoundTstDlgClientRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{SSound Implementation}}

SSound::SSound(TWindow *parent, const char *waveFN, TModule *module)
: TWindow(parent, waveFN, module)
{
	WaveFilename = new char[strlen(ExePath)+strlen(waveFN)+1];
   strcpy(WaveFilename, ExePath);
   strcat(WaveFilename, waveFN);

   SoundId = 0;
   NotifyHWindow = 0;   
   IsCreated = false;
   DebugMode = false;
}


SSound::~SSound()
{
   ::PostMessage(GLOBAL_HWndCmsSound, WM_CMSSOUND_FREESOUNDID, (WPARAM)SoundId, (LPARAM)0);

	delete[] WaveFilename;
}


void SSound::SetupWindow()
{
	if (IsCreated) return;
   IsCreated = true;

	TWindow::SetupWindow();
//
// If no pointer to CmsSoundWindow then ask the OS for it
//
   if (!GLOBAL_HWndCmsSound) {
   	GLOBAL_HWndCmsSound = FindWindow("OWL_Window", CMSSOUND_WINDOWNAME);
   }
//
// CmsSoundWindow does not exist - execute
//
   if (!GLOBAL_HWndCmsSound) {
	   unsigned int size = strlen(ExePath)+1;
		char *path = new char[size];

      strcpy(path, ExePath);
	//
   // Locate the file within the path
   //
   	if (LocateFileWithinPath(CMSSOUND_EXEFILENAME, path, size)) {
		   char *exeCommand = new char[strlen(path)+strlen(CMSSOUND_EXEFILENAME)+30+strlen(WaveFilename)+1];

         int nCmdShow = SW_HIDE;
         if (DebugMode) {
				nCmdShow = SW_SHOW;
         }
			sprintf(exeCommand, "\"%s%s\" %ld \"%s\" %d", path, CMSSOUND_EXEFILENAME, (long)HWindow, WaveFilename, nCmdShow);

         WinExec(exeCommand, nCmdShow);

         delete[] exeCommand;
      }

      delete[] path;
//
// CmsSoundWindow exists - request a new ID
//
   } else {
   	if (SendFilename(WaveFilename, strlen(WaveFilename)+1)) {
	   	::SendMessage(GLOBAL_HWndCmsSound, WM_CMSSOUND_REQUESTSOUNDID, (WPARAM)HWindow, (LPARAM)0);
      }
   }
}


int SSound::Play(const int flags)
{
//
// Play the wave file
//
   return (int)::SendMessage(GLOBAL_HWndCmsSound, WM_CMSSOUND_PLAY, (WPARAM)SoundId,
   								  (LPARAM)flags);
}


int SSound::Stop(const int flags)
{
   return (int)::SendMessage(GLOBAL_HWndCmsSound, WM_CMSSOUND_STOP, (WPARAM)SoundId,
   								  (LPARAM)flags);
}


int SSound::StopAll(void)
{
	return (int)::SendMessage(GLOBAL_HWndCmsSound, WM_CMSSOUND_STOPALL, (WPARAM)0, (LPARAM)0);
}


DWORD SSound::GetErrorCode(void)
{
	return (int)::SendMessage(GLOBAL_HWndCmsSound, WM_CMSSOUND_GETLASTERROR, (WPARAM)SoundId,
   								  (LPARAM)0);
}


unsigned int SSound::GetDuration(void)
{
	return (unsigned int)::SendMessage(GLOBAL_HWndCmsSound, WM_CMSSOUND_GETDURATION,
   											  (WPARAM)SoundId, (LPARAM)0);
}


unsigned int SSound::GetTime(void)
{
	return (unsigned int)::SendMessage(GLOBAL_HWndCmsSound, WM_CMSSOUND_GETPOSITION,
   											  (WPARAM)SoundId, (LPARAM)0);
}


/*
int SSound::ShowError(const bool show)
{
	if (!show) return ErrCode;

   switch (ErrCode) {
   	case 0:	break;

   	default:	{
      				char *buf = new char[256];

				      if (!mciGetErrorString(ErrCode, buf, 255)) {
      	            sprintf(buf, "Unknown error code %d detected in SSound class", ErrCode);
   	            }
						::MessageBox(0, buf, "SSound Class Error", MB_OK | MB_ICONWARNING | MB_APPLMODAL);

                  delete[] buf;
      			}
               break;
   }

   return ErrCode;
}
*/


bool SSound::SetNotifyComplete(HWND hWnd, const int msg, const WPARAM wParam, const LPARAM lParam)
{
	NotifyHWindow = hWnd;
   NotifyMessage = msg;
   NotifyWParam = wParam;
   NotifyLParam = lParam;

   return true;
}


bool SSound::IsPlaying(void)
{
	return (bool)::SendMessage(GLOBAL_HWndCmsSound, WM_CMSSOUND_ISPLAYING, (WPARAM)SoundId, 0);
}


bool SSound::IsSoundPlaying(void)
{
	return (bool)::SendMessage(GLOBAL_HWndCmsSound, WM_CMSSOUND_ISSOUNDPLAYING, 0, 0);
}


LRESULT SSound::WmCmsSndInitContact(WPARAM id, LPARAM hWnd)
{
	GLOBAL_HWndCmsSound = (HWND)hWnd;
   SoundId = (unsigned int)id;

   return true;
}


LRESULT SSound::WmNotifyWhenPlayFinished(WPARAM /*id*/, LPARAM)
{
	if (NotifyHWindow) {
   	::PostMessage(NotifyHWindow, NotifyMessage, NotifyWParam, NotifyLParam);
   }

   return true;
}


LRESULT SSound::WmNotifyWhenStopped(WPARAM /*id*/, LPARAM)
{
	if (NotifyHWindow) {
   	::PostMessage(NotifyHWindow, NotifyMessage, NotifyWParam, NotifyLParam);
   }

   return true;
}


LRESULT SSound::WmPing(WPARAM, LPARAM)
{
	return 0xffffffff;		// Return -1
}


bool SSound::SendFilename(const char *data, const unsigned int size)
{
   COPYDATASTRUCT copyData;

   copyData.cbData = (DWORD)size;
   copyData.lpData = (PVOID)data;
//
// Initiate start (sending the length of the string)
//
   return ::SendMessage(GLOBAL_HWndCmsSound, WM_COPYDATA, (WPARAM)HWindow, (LPARAM)&copyData);
}


void SSound::SetDebug(const bool debug)
{
	DebugMode = debug;
}
